-- ----------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Entity Configurable_XMEM Is
-- ----------------------------------------------------------------------------
Port (
    STB_I       : In    Std_Logic;
    CYC_I       : In    Std_Logic;
    ACK_O       : Out   Std_Logic;
    ADR_I       : In    Std_Logic_Vector(25-1 Downto 0);
    DAT_O       : Out   Std_Logic_Vector(31 Downto 0);
    DAT_I       : In    Std_Logic_Vector(31 Downto 0);
    SEL_I       : In    Std_Logic_Vector( 3 Downto 0);
    WE_I        : In    Std_Logic;
    CLK_I       : In    Std_Logic;
    RST_I       : In    Std_Logic;

    SDRAM_CLK     : In    Std_Logic;

    -- 1x32 wide device
 
 
 
 
 
 
 
 
 
    -- 1x16 wide device
    SDRAM_A          : Out   Std_Logic_Vector(13-1 Downto 0); -- IncludeIf_1_Device_X_16
    SDRAM_BA         : Out   Std_Logic_Vector(1  Downto 0);                         -- IncludeIf_1_Device_X_16
    SDRAM_BE         : Out   Std_Logic_Vector(1  Downto 0);                         -- IncludeIf_1_Device_X_16
    SDRAM_CAS        : Out   Std_Logic;                                             -- IncludeIf_1_Device_X_16
    SDRAM_CLKE       : Out   Std_Logic;                                             -- IncludeIf_1_Device_X_16
    SDRAM_CS         : Out   Std_Logic;                                             -- IncludeIf_1_Device_X_16
    SDRAM_D          : InOut Std_Logic_Vector(15 Downto 0);                         -- IncludeIf_1_Device_X_16
    SDRAM_RAS        : Out   Std_Logic;                                             -- IncludeIf_1_Device_X_16
    SDRAM_WE         : Out   Std_Logic                                              -- IncludeIf_1_Device_X_16
    -- 1x8 wide device
 
 
 
 
 
 
 
 
 
    -- 2x16 wide device
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 

 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
);
End Configurable_XMEM;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Architecture RTL Of Configurable_XMEM Is
-- ----------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Component WB_SHARED_SDRAM Is
        Port (
            -- Timing configuration
            CONFIG_BURST_LENGTH                  : In  Std_Logic_Vector( 2 Downto 0);
            CONFIG_TIMING_TCL                    : In  Std_Logic_Vector( 1 Downto 0);
            CONFIG_TIMING_TMRD                   : In  Std_Logic_Vector( 2 Downto 0);
            CONFIG_TIMING_TRCD                   : In  Std_Logic_Vector( 2 Downto 0);
            CONFIG_TIMING_TRFC                   : In  Std_Logic_Vector( 2 Downto 0);
            CONFIG_TIMING_TRP                    : In  Std_Logic_Vector( 2 Downto 0);
            CONFIG_TIMING_TWR                    : In  Std_Logic_Vector( 2 Downto 0);
            REFRESH_INTERVAL                     : In  Std_Logic_Vector( 2 Downto 0);
            REFRESH_PRESCALER                    : In  Std_Logic_Vector( 7 Downto 0);

            --SRAM interface
            CONTROL_WB_SRAM_ACCESS_GRANTED       : Out Std_Logic;
            CONTROL_WB_SRAM_ACCESS_REQUEST       : In  Std_Logic;
            CONTROL_WB_SRAM_ADR                  : In  Std_Logic_Vector(23 Downto 0);
            CONTROL_WB_SRAM_BE                   : In  Std_Logic_Vector( 3 Downto 0);
            CONTROL_WB_SRAM_D_EN                 : In  Std_Logic_Vector(31 Downto 0);
            CONTROL_WB_SRAM_DAT_IN               : In  Std_Logic_Vector(31 Downto 0);
            CONTROL_WB_SRAM_DAT_OUT              : Out Std_Logic_Vector(31 Downto 0);
            CONTROL_WB_SRAM_OE                   : In  Std_Logic;
            CONTROL_WB_SRAM_TRANSFER             : In  Std_Logic;
            CONTROL_WB_SRAM_TRANSFER_DONE        : In  Std_Logic;
            CONTROL_WB_SRAM_W                    : In  Std_Logic;

            -- SDRAM interface
            CONTROL_WB_SDRAM_ACKNOWLEDGE         : In  Std_Logic;
            CONTROL_WB_SDRAM_BANK_ADDRESS        : In  Std_Logic_Vector( 1 Downto 0);
            CONTROL_WB_SDRAM_COLUMN_ADDRESS      : In  Std_Logic_Vector(12 Downto 0);
            CONTROL_WB_SDRAM_DATA_FROM_MEM       : Out Std_Logic_Vector(31 Downto 0);
            CONTROL_WB_SDRAM_DATA_TO_MEM         : In  Std_Logic_Vector(31 Downto 0);
            CONTROL_WB_SDRAM_MEM_BE              : In  Std_Logic_Vector( 3 Downto 0);
            CONTROL_WB_SDRAM_REQUEST_READ        : In  Std_Logic;
            CONTROL_WB_SDRAM_REQUEST_TRANSFER    : In  Std_Logic;
            CONTROL_WB_SDRAM_ROW_ADDRESS         : In  Std_Logic_Vector(12 Downto 0);
            CONTROL_WB_SDRAM_SDRAM_MODE_REGISTER : In  Std_Logic_Vector(12 Downto 0);
            CONTROL_WB_SDRAM_TRANSFER_DONE       : Out Std_Logic;

            -- FLASH interface
            CONTROL_WB_FLASH_ACCESS_GRANTED      : Out Std_Logic;
            CONTROL_WB_FLASH_ACCESS_REQUEST      : In  Std_Logic;
            CONTROL_WB_FLASH_ADR                 : In  Std_Logic_Vector(23 Downto 0);
            CONTROL_WB_FLASH_BE                  : In  Std_Logic_Vector( 3 Downto 0);
            CONTROL_WB_FLASH_BUSY                : Out Std_Logic;
            CONTROL_WB_FLASH_D_EN                : In  Std_Logic_Vector(31 Downto 0);
            CONTROL_WB_FLASH_DAT_IN              : In  Std_Logic_Vector(31 Downto 0);
            CONTROL_WB_FLASH_DAT_OUT             : Out Std_Logic_Vector(31 Downto 0);
            CONTROL_WB_FLASH_OE                  : In  Std_Logic;
            CONTROL_WB_FLASH_RESET               : In  Std_Logic;
            CONTROL_WB_FLASH_TRANSFER            : In  Std_Logic;
            CONTROL_WB_FLASH_TRANSFER_DONE       : In  Std_Logic;
            CONTROL_WB_FLASH_W                   : In  Std_Logic;

            -- External Memory interface
            MEM_A                                : Out Std_Logic_Vector(23 Downto 0);
            MEM_BE                               : Out Std_Logic_Vector( 3 Downto 0);
            MEM_CLK                              : In  Std_Logic;
            MEM_D_EN                             : Out Std_Logic_Vector(31 Downto 0);
            MEM_D_IN                             : In  Std_Logic_Vector(31 Downto 0);
            MEM_D_OUT                            : Out Std_Logic_Vector(31 Downto 0);
            MEM_FLASH_BUSY                       : In  Std_Logic;
            MEM_FLASH_E                          : Out Std_Logic;
            MEM_FLASH_RESET                      : Out Std_Logic;
            MEM_OE                               : Out Std_Logic;
            MEM_RST                              : In  Std_Logic;
            MEM_SDRAM_CAS                        : Out Std_Logic;
            MEM_SDRAM_CKE                        : Out Std_Logic;
            MEM_SDRAM_E                          : Out Std_Logic;
            MEM_SDRAM_RAS                        : Out Std_Logic;
            MEM_SRAM_E                           : Out Std_Logic;
            MEM_W                                : Out Std_Logic
        );
    End Component;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Signal Signal_GND            : Std_Logic;
    Signal Signal_GND4           : Std_Logic_Vector( 3 Downto 0);
    Signal Signal_GND24          : Std_Logic_Vector(23 Downto 0);
    Signal Signal_GND32          : Std_Logic_Vector(31 Downto 0);

    Signal Signal_VCC            : Std_Logic;
    Signal Signal_VCC4           : Std_Logic_Vector( 3 Downto 0);
    Signal Signal_VCC24          : Std_Logic_Vector(23 Downto 0);
    Signal Signal_VCC32          : Std_Logic_Vector(31 Downto 0);

    Signal Port_MEM_A            : Std_Logic_Vector(23 Downto 0);
    Signal Port_MEM_BE           : Std_Logic_Vector( 3 Downto 0);
    Signal Port_MEM_Data_Enable  : Std_Logic_Vector(31 Downto 0);
    Signal Port_MEM_Data_Input   : Std_Logic_Vector(31 Downto 0);
    Signal Port_MEM_Data_Output  : Std_Logic_Vector(31 Downto 0);
    Signal Port_MEM_FLASH_Enable : Std_Logic;
    Signal Port_MEM_FLASH_Reset  : Std_Logic;
    Signal Port_MEM_OutputEnable : Std_Logic;
    Signal Port_MEM_SDRAM_CAS    : Std_Logic;
    Signal Port_MEM_SDRAM_CKE    : Std_Logic;
    Signal Port_MEM_SDRAM_E      : Std_Logic;
    Signal Port_MEM_SDRAM_RAS    : Std_Logic;
    Signal Port_MEM_SRAM_E       : Std_Logic;
    Signal Port_MEM_W            : Std_Logic;
    -- ------------------------------------------------------------------------
    -- ------------------------------------------------------------------------
    Signal Config_Active_To_ReadWrite_Value : Std_Logic_Vector(2  Downto 0);
    Signal Config_Auto_Refresh_Period_Value : Std_Logic_Vector(2  Downto 0);
    Signal Config_Burst_Length              : Std_Logic_Vector(2  Downto 0);
    Signal Config_Burst_Mode                : Std_Logic;
    Signal Config_Burst_Type                : Std_Logic;
    Signal Config_Bus_Width                 : Std_Logic_Vector(1  Downto 0);
    Signal Config_Cl_Value                  : Std_Logic_Vector(1  Downto 0);
    Signal Config_Keep_Row_Open             : Std_Logic;
    Signal Config_Mem_Size                  : Std_Logic_Vector(1  Downto 0);
    Signal Config_Precharge_Period_Value    : Std_Logic_Vector(2  Downto 0);
    Signal Config_Refresh_Interval          : Std_Logic_Vector(2  Downto 0);
    Signal Config_Refresh_Prescaler         : Std_Logic_Vector(7  Downto 0);
    Signal Config_Write_Completion_Value    : Std_Logic_Vector(2  Downto 0);
    -- ------------------------------------------------------------------------


    Signal SDRAMWishbone_DataInput                       : Std_Logic_Vector(31 Downto 0);
    Signal SDRAMWishbone_DataOutput                      : Std_Logic_Vector(31 Downto 0);
    Signal SDRAMWishbone_Address                         : Std_Logic_Vector(25 Downto 0);
    Signal SDRAMWishbone_ByteSelect                      : Std_Logic_Vector( 3 Downto 0);
    Signal SDRAMWishbone_RequestRead                     : Std_Logic;
    Signal SDRAMWishbone_RequestWrite                    : Std_Logic;
    Signal SDRAMWishbone_RequestTransfer                 : Std_Logic;
    Signal SDRAMWishbone_Acknowledge                     : Std_Logic;
    Signal SDRAMWishbone_TransferDone                    : Std_Logic;
    Signal SDRAMWishbone_TransferDone_MetaStableFF       : Std_Logic;
    Signal SDRAMWishbone_TransferDone_ResyncFF0          : Std_Logic;
    Signal SDRAMWishbone_TransferDone_ResyncFF1          : Std_Logic;
    Signal SDRAMControl_TransferDone                     : Std_Logic;

    Signal SDRAMControl_RequestTransfer_MetastableFF     : Std_Logic;
    Signal SDRAMControl_RequestTransfer_ResyncFF0        : Std_Logic;
    Signal SDRAMControl_RequestTransfer_ResyncFF1        : Std_Logic;
    Signal SDRAMControl_RequestTransfer                  : Std_Logic;
    Signal SDRAMControl_WishboneAcknowledge              : Std_Logic;
    Signal SDRAMControl_WishboneAcknowledge_MetastableFF : Std_Logic;
    Signal SDRAMControl_WishboneAcknowledge_ResyncFF0    : Std_Logic;
    Signal SDRAMControl_WishboneAcknowledge_ResyncFF1    : Std_Logic;
    Signal SDRAMControl_TransferDone_ResyncFF0           : Std_Logic;
    Signal SdramWishbone_RowAddress                      : Std_Logic_Vector(12 Downto 0);
    Signal SdramWishbone_ColumnAddress                   : Std_Logic_Vector(12 Downto 0);
    Signal SdramWishbone_BankAddress                     : Std_Logic_Vector( 1 Downto 0);
    Signal SdramWishbone_ModeRegister                    : Std_Logic_Vector(12 Downto 0);

-- ----------------------------------------------------------------------------
Begin
-- ----------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Config_Active_To_ReadWrite_Value    <= "001";
    Config_Auto_Refresh_Period_Value    <= "110";
    Config_Burst_Length                 <= "001";
    Config_Burst_Mode                   <= '0';
    Config_Burst_Type                   <= '0';
    Config_Bus_Width                    <= "01";
    Config_Cl_Value                     <= "10";
    Config_Keep_Row_Open                <= '1';
    Config_Mem_Size                     <= "11";
    Config_Precharge_Period_Value       <= "001";
--    Config_Refresh_Interval             <= "110";
--    Config_Refresh_Prescaler            <= "11011100";
    Config_Refresh_Interval             <= "010";
    Config_Refresh_Prescaler            <= "00110010";
    Config_Write_Completion_Value       <= "001";

    -- ------------------------------------------------------------------------


    -- ------------------------------------------------------------------------
    Signal_GND   <= '0';
    Signal_GND4  <= (Others=>'0');
    Signal_GND24 <= (Others=>'0');
    Signal_GND32 <= (Others=>'0');

    Signal_VCC   <= '1';
    Signal_VCC4  <= (Others=>'1');
    Signal_VCC24 <= (Others=>'1');
    Signal_VCC32 <= (Others=>'1');
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    SubPart_MemoryController : WB_SHARED_SDRAM Port Map (
            -- Timing configuration
            CONFIG_BURST_LENGTH                  => Config_Burst_Length,
            CONFIG_TIMING_TCL                    => Config_Cl_Value,
            CONFIG_TIMING_TMRD                   => Config_Active_To_ReadWrite_Value, -- TODO
            CONFIG_TIMING_TRCD                   => Config_Active_To_ReadWrite_Value,
            CONFIG_TIMING_TRFC                   => Config_Auto_Refresh_Period_Value,
            CONFIG_TIMING_TRP                    => Config_Precharge_Period_Value,
            CONFIG_TIMING_TWR                    => Config_Write_Completion_Value,
            REFRESH_INTERVAL                     => Config_Refresh_Interval,
            REFRESH_PRESCALER                    => Config_Refresh_Prescaler,

            --SRAM interface
            CONTROL_WB_SRAM_ACCESS_GRANTED       => Open,
            CONTROL_WB_SRAM_ACCESS_REQUEST       => Signal_GND,
            CONTROL_WB_SRAM_ADR                  => Signal_GND24,
            CONTROL_WB_SRAM_BE                   => Signal_VCC4,
            CONTROL_WB_SRAM_D_EN                 => Signal_VCC32,
            CONTROL_WB_SRAM_DAT_IN               => Signal_GND32,
            CONTROL_WB_SRAM_DAT_OUT              => Open,
            CONTROL_WB_SRAM_OE                   => Signal_VCC,
            CONTROL_WB_SRAM_TRANSFER             => Signal_GND,
            CONTROL_WB_SRAM_TRANSFER_DONE        => Signal_VCC,
            CONTROL_WB_SRAM_W                    => Signal_VCC,

            -- SDRAM interface
            CONTROL_WB_SDRAM_ACKNOWLEDGE         => SDRAMControl_WishboneAcknowledge,
            CONTROL_WB_SDRAM_BANK_ADDRESS        => SdramWishbone_BankAddress,
            CONTROL_WB_SDRAM_COLUMN_ADDRESS      => SdramWishbone_ColumnAddress,
            CONTROL_WB_SDRAM_DATA_FROM_MEM       => SDRAMWishbone_DataOutput,
            CONTROL_WB_SDRAM_DATA_TO_MEM         => SDRAMWishbone_DataInput,
            CONTROL_WB_SDRAM_MEM_BE              => SDRAMWishbone_ByteSelect,
            CONTROL_WB_SDRAM_REQUEST_READ        => SDRAMWishbone_RequestRead,
            CONTROL_WB_SDRAM_REQUEST_TRANSFER    => SDRAMControl_RequestTransfer,
            CONTROL_WB_SDRAM_ROW_ADDRESS         => SdramWishbone_RowAddress,
            CONTROL_WB_SDRAM_SDRAM_MODE_REGISTER => SdramWishbone_ModeRegister,
            CONTROL_WB_SDRAM_TRANSFER_DONE       => SDRAMControl_TransferDone,

            -- FLASH interface
            CONTROL_WB_FLASH_ACCESS_GRANTED      => Open,
            CONTROL_WB_FLASH_ACCESS_REQUEST      => Signal_GND,
            CONTROL_WB_FLASH_ADR                 => Signal_GND24,
            CONTROL_WB_FLASH_BE                  => Signal_VCC4,
            CONTROL_WB_FLASH_BUSY                => Open,
            CONTROL_WB_FLASH_D_EN                => Signal_VCC32,
            CONTROL_WB_FLASH_DAT_IN              => Signal_GND32,
            CONTROL_WB_FLASH_DAT_OUT             => Open,
            CONTROL_WB_FLASH_OE                  => Signal_VCC,
            CONTROL_WB_FLASH_RESET               => Signal_VCC,
            CONTROL_WB_FLASH_TRANSFER            => Signal_GND,
            CONTROL_WB_FLASH_TRANSFER_DONE       => Signal_VCC,
            CONTROL_WB_FLASH_W                   => Signal_VCC,

            -- External Memory interface
            MEM_A                                => Port_MEM_A,            --: Out Std_Logic_Vector(23 Downto 0);
            MEM_BE                               => Port_MEM_BE,           --: Out Std_Logic_Vector( 3 Downto 0);
            MEM_CLK                              => SDRAM_CLK,
            MEM_D_EN                             => Port_MEM_Data_Enable,  --: Out Std_Logic_Vector(31 Downto 0);
            MEM_D_IN                             => Port_MEM_Data_Input,   --: In  Std_Logic_Vector(31 Downto 0);
            MEM_D_OUT                            => Port_MEM_Data_Output,  --: Out Std_Logic_Vector(31 Downto 0);
            MEM_FLASH_BUSY                       => Signal_VCC,            --: In  Std_Logic;
            MEM_FLASH_E                          => Port_MEM_FLASH_Enable, --: Out Std_Logic;
            MEM_FLASH_RESET                      => Port_MEM_FLASH_Reset,  --: Out Std_Logic;
            MEM_OE                               => Port_MEM_OutputEnable, --: Out Std_Logic;
            MEM_RST                              => RST_I,
            MEM_SDRAM_CAS                        => Port_MEM_SDRAM_CAS,    --: Out Std_Logic;
            MEM_SDRAM_CKE                        => Port_MEM_SDRAM_CKE,    --: Out Std_Logic;
            MEM_SDRAM_E                          => Port_MEM_SDRAM_E,      --: Out Std_Logic;
            MEM_SDRAM_RAS                        => Port_MEM_SDRAM_RAS,    --: Out Std_Logic;
            MEM_SRAM_E                           => Port_MEM_SRAM_E,       --: Out Std_Logic;
            MEM_W                                => Port_MEM_W             --: Out Std_Logic
    );
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(CLK_I)
    Begin
        If Rising_Edge(CLK_I) Then
            If RST_I='1' Then
                DAT_O                 <= (Others=>'0');
                SDRAMWishbone_DataInput                 <= (Others=>'0');
                SDRAMWishbone_Address                   <= (Others=>'0');
                SDRAMWishbone_ByteSelect                <= (Others=>'0');
                SDRAMWishbone_RequestRead               <= '0';
                SDRAMWishbone_RequestWrite               <= '0';
                SDRAMWishbone_RequestTransfer           <= '0';
                SDRAMWishbone_Acknowledge               <= '0';
                SDRAMWishbone_TransferDone              <= '0';
                SDRAMWishbone_TransferDone_MetaStableFF <= '0';
                SDRAMWishbone_TransferDone_ResyncFF0    <= '0';
                SDRAMWishbone_TransferDone_ResyncFF1    <= '0';
             Else
                 If SDRAMControl_TransferDone='1' Then
                     DAT_O <= SDRAMWishbone_DataOutput;
                 End If;
                 SDRAMWishbone_DataInput  <= DAT_I;
                 SDRAMWishbone_Address     <= '0' & ADR_I;
    -- ------------------------------------------------------------------------
                 SDRAMWishbone_ByteSelect <= SEL_I;
                 SDRAMWishbone_RequestWrite <= WE_I;
                 SDRAMWishbone_RequestRead <= Not WE_I;
                 SDRAMWishbone_TransferDone_MetaStableFF <= SDRAMControl_TransferDone;
                 SDRAMWishbone_TransferDone_ResyncFF0    <= SDRAMWishbone_TransferDone_MetaStableFF;
                 SDRAMWishbone_TransferDone_ResyncFF1    <= SDRAMWishbone_TransferDone_ResyncFF0;
                 SDRAMWishbone_RequestTransfer           <= CYC_I   And STB_I And Not SDRAMWishbone_Acknowledge;
                 SDRAMWishbone_Acknowledge               <= SDRAMWishbone_TransferDone_MetaStableFF And
                                                            SDRAMWishbone_TransferDone_ResyncFF0    And
                                                            Not SDRAMWishbone_TransferDone_ResyncFF1;
             End If;
         End If;
     End Process;
 
                     ACK_O <= SDRAMWishbone_Acknowledge;
    Process(SDRAM_CLK)
    Begin
        If Rising_Edge(SDRAM_CLK) Then
            If RST_I='1' Then
                SDRAMControl_RequestTransfer_MetastableFF      <= '0';
                SDRAMControl_RequestTransfer_ResyncFF0         <= '0';
                SDRAMControl_RequestTransfer_ResyncFF1         <= '0';
                SDRAMControl_RequestTransfer                   <= '0';
                SDRAMControl_WishboneAcknowledge               <= '0';
                SDRAMControl_WishboneAcknowledge_MetastableFF  <= '0';
                SDRAMControl_WishboneAcknowledge_ResyncFF0     <= '0';
                SDRAMControl_WishboneAcknowledge_ResyncFF1     <= '0';
             Else
                SDRAMControl_RequestTransfer_MetastableFF     <= SDRAMWishbone_RequestTransfer;
                SDRAMControl_RequestTransfer_ResyncFF0        <= SDRAMControl_RequestTransfer_MetastableFF;
                SDRAMControl_RequestTransfer_ResyncFF1        <= SDRAMControl_RequestTransfer_ResyncFF0;
                SDRAMControl_WishboneAcknowledge_MetastableFF <= SDRAMWishbone_Acknowledge;
                SDRAMControl_WishboneAcknowledge_ResyncFF0    <= SDRAMControl_WishboneAcknowledge_MetastableFF;
                SDRAMControl_WishboneAcknowledge_ResyncFF1    <= SDRAMControl_WishboneAcknowledge_ResyncFF0;
                SDRAMControl_RequestTransfer                  <= SDRAMControl_RequestTransfer_MetastableFF              And
                                                                 SDRAMControl_RequestTransfer_ResyncFF0    And
                                                                 Not SDRAMControl_RequestTransfer_ResyncFF1;
                SDRAMControl_WishboneAcknowledge              <= SDRAMControl_WishboneAcknowledge_ResyncFF0              And
                                                                 Not SDRAMControl_WishboneAcknowledge_ResyncFF1;
             End If;
         End If;
     End Process;
 

    -- ------------------------------------------------------------------------
SdramWishbone_RowAddress    <= SdramWishbone_Address(22 Downto 10);
SdramWishbone_ColumnAddress <= "0000" & SdramWishbone_Address(9 Downto 2) & '0' ;
SdramWishbone_BankAddress   <= SdramWishbone_Address(24 Downto 23);
SdramWishbone_ModeRegister  <= "000" & Config_Burst_Mode & "000" & Config_Cl_Value & Config_Burst_Type & Config_Burst_Length;
SDRAM_D(0) <= Port_MEM_Data_Output(0) When Port_MEM_Data_Enable(0) = '0' Else 'Z';
Port_MEM_Data_Input(0) <= SDRAM_D(0);
SDRAM_D(1) <= Port_MEM_Data_Output(1) When Port_MEM_Data_Enable(1) = '0' Else 'Z';
Port_MEM_Data_Input(1) <= SDRAM_D(1);
SDRAM_D(2) <= Port_MEM_Data_Output(2) When Port_MEM_Data_Enable(2) = '0' Else 'Z';
Port_MEM_Data_Input(2) <= SDRAM_D(2);
SDRAM_D(3) <= Port_MEM_Data_Output(3) When Port_MEM_Data_Enable(3) = '0' Else 'Z';
Port_MEM_Data_Input(3) <= SDRAM_D(3);
SDRAM_D(4) <= Port_MEM_Data_Output(4) When Port_MEM_Data_Enable(4) = '0' Else 'Z';
Port_MEM_Data_Input(4) <= SDRAM_D(4);
SDRAM_D(5) <= Port_MEM_Data_Output(5) When Port_MEM_Data_Enable(5) = '0' Else 'Z';
Port_MEM_Data_Input(5) <= SDRAM_D(5);
SDRAM_D(6) <= Port_MEM_Data_Output(6) When Port_MEM_Data_Enable(6) = '0' Else 'Z';
Port_MEM_Data_Input(6) <= SDRAM_D(6);
SDRAM_D(7) <= Port_MEM_Data_Output(7) When Port_MEM_Data_Enable(7) = '0' Else 'Z';
Port_MEM_Data_Input(7) <= SDRAM_D(7);
SDRAM_D(8) <= Port_MEM_Data_Output(8) When Port_MEM_Data_Enable(8) = '0' Else 'Z';
Port_MEM_Data_Input(8) <= SDRAM_D(8);
SDRAM_D(9) <= Port_MEM_Data_Output(9) When Port_MEM_Data_Enable(9) = '0' Else 'Z';
Port_MEM_Data_Input(9) <= SDRAM_D(9);
SDRAM_D(10) <= Port_MEM_Data_Output(10) When Port_MEM_Data_Enable(10) = '0' Else 'Z';
Port_MEM_Data_Input(10) <= SDRAM_D(10);
SDRAM_D(11) <= Port_MEM_Data_Output(11) When Port_MEM_Data_Enable(11) = '0' Else 'Z';
Port_MEM_Data_Input(11) <= SDRAM_D(11);
SDRAM_D(12) <= Port_MEM_Data_Output(12) When Port_MEM_Data_Enable(12) = '0' Else 'Z';
Port_MEM_Data_Input(12) <= SDRAM_D(12);
SDRAM_D(13) <= Port_MEM_Data_Output(13) When Port_MEM_Data_Enable(13) = '0' Else 'Z';
Port_MEM_Data_Input(13) <= SDRAM_D(13);
SDRAM_D(14) <= Port_MEM_Data_Output(14) When Port_MEM_Data_Enable(14) = '0' Else 'Z';
Port_MEM_Data_Input(14) <= SDRAM_D(14);
SDRAM_D(15) <= Port_MEM_Data_Output(15) When Port_MEM_Data_Enable(15) = '0' Else 'Z';
Port_MEM_Data_Input(15) <= SDRAM_D(15);
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
--    ReplaceWith_SDRAM_DataPortBinding
    -- ------------------------------------------------------------------------



    -- ------------------------------------------------------------------------
    -- 1x32 wide device
 
 
 
 
 
 
 
 
 

    -- ------------------------------------------------------------------------
    -- 1x16 wide device
    SDRAM_A          <= Port_MEM_A(13 Downto 1);    --: Out   Std_Logic_Vector(13-1 Downto 0);                       -- IncludeIf_1_Device_X_16
    SDRAM_BA         <= Port_MEM_A(15 Downto 14);   --: Out   Std_Logic_Vector(1  Downto 0);                         -- IncludeIf_1_Device_X_16
    SDRAM_BE         <= Port_MEM_BE(1 Downto 0);    --: Out   Std_Logic_Vector(1  Downto 0);                         -- IncludeIf_1_Device_X_16
    SDRAM_CAS        <= Port_MEM_SDRAM_CAS;         --: Out   Std_Logic;                                             -- IncludeIf_1_Device_X_16
    SDRAM_CLKE       <= Port_MEM_SDRAM_CKE;         --: Out   Std_Logic;                                             -- IncludeIf_1_Device_X_16
    SDRAM_CS         <= Port_MEM_SDRAM_E;           --: Out   Std_Logic;                                             -- IncludeIf_1_Device_X_16
--    SDRAM_D          : InOut Std_Logic_Vector(15 Downto 0);                                                          -- IncludeIf_1_Device_X_16
    SDRAM_RAS        <= Port_MEM_SDRAM_RAS;         --: Out   Std_Logic;                                             -- IncludeIf_1_Device_X_16
    SDRAM_WE         <= Port_MEM_W;                 --: Out   Std_Logic                                              -- IncludeIf_1_Device_X_16

    -- ------------------------------------------------------------------------
    -- 1x8 wide device
 
 
 
 
 
 
 
 
 

    -- ------------------------------------------------------------------------
    -- 2x16 wide device
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 

    -- ------------------------------------------------------------------------
    -- 2x8 wide device
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 



-- ----------------------------------------------------------------------------
End RTL;
-- ----------------------------------------------------------------------------

